#-------------------------------------BOF---------------------------------------
#
# File      : hexagon.py
# Date      : May 16, 2010
# Created by: Bartosz Gradzik "akaBarten"
#             www.eng-tips.com
#
#-------------------------------------------------------------------------------
#
# Do not delete the following import lines
from abaqus import *
from abaqusConstants import *
import __main__
#
#-------------------------------------------------------------------------------
#   U S E R  P A R A M E T E R S
#-------------------------------------------------------------------------------
#
user_inputFile = "C:\\Users\\Bartosz\\Desktop\\small_hexagon.dat"
#user_inputFile = "C:\\Users\\Bartosz\\Desktop\\big_hexagon.dat"
user_length = 200.0
user_splitSize = 1
#
# Comments:
# 1. user_length -> distance between points in hexagon
# 2. user_splitSize -> how many times each hexagon edge should be spilt
#                      number of connectors on a edge is equal 2^user_splitSize
#                      user_splitSize = 0 --> 2^0=1 --> 1 connector element
#                      user_splitSize = 1 --> 2^1=2 --> 2 connector element
#                      user_splitSize = 2 --> 2^2=4 --> 4 connector element
#
#-------------------------------------------------------------------------------
#
def readDataPoints(fileName):
    """
    Read data points from ASCII file.
    Correct format:
    x-coord, y-coord
    x-coord, y-coord
    ...    ,     ...
    x-coord, y-coord
    """
    #
    # open and read from a file
    inputFile = open(fileName,'r')
    inputDataLines = inputFile.readlines()
    #
    outputCurveLines = []
    for line in inputDataLines:
        line = string.split(line,',')
        PairData = (string.atof(line[0]),string.atof(line[1]))
        outputCurveLines.append(PairData)
    return outputCurveLines
#
#-------------------------------------------------------------------------------
#
def length(p1, p2):
    """
    Calculate distance between two points in 2D plane.
    """
    length = abs(sqrt((p2[0]-p1[0])**2+(p2[1]-p1[1])**2))
    return length
#
#-------------------------------------------------------------------------------
#
def findEdge(points=None,hexLength=None):
    """
    Define connectivity for hexagon.
    A point is connected with other points if distance
    is smaller then 'hexLength'+ 10% of hexLength.
    """
    edge = []
    numberOfPoints = len(points)
    #
    for i in range(0,numberOfPoints-1):
        for j in range(i+1,numberOfPoints):
            if length(points[i], points[j]) <= (hexLength+(hexLength*0.1)):
               edge.append((points[i],points[j]))
    return edge
#
#-------------------------------------------------------------------------------
#
def hexagon(points=None, splitSize=None):
    """
    Generate cae model.
    """
    #
    import section
    import regionToolset
    import displayGroupMdbToolset as dgm
    import part
    import material
    import assembly
    import step
    import load
    import mesh
    import job
    import sketch
    import visualization
    import displayGroupOdbToolset as dgo
    #
    #------------------------------------
    # S K E T C H
    #------------------------------------
    #
    # create a new sketch
    mySketch = mdb.models['Model-1'].ConstrainedSketch(name='hexagon',sheetSize=200.0)
    #
    # connect hexagon points with lines
    for edge in points:
        mySketch.Line(point1=edge[0], point2=edge[1])
    mySketch.unsetPrimaryObject()
    #
    #------------------------------------
    # P A R T
    #------------------------------------
    #
    # create a new part
    myPart = mdb.models['Model-1'].Part(name='Hexagon', dimensionality=TWO_D_PLANAR, type=DEFORMABLE_BODY)
    myPart.BaseWire(sketch=mySketch)
    #
    # split hexagon edges
    for i in range(splitSize):
        myPart.PartitionEdgeByParam(edges=myPart.edges, parameter=0.5)
    #
    # build wires (need to connectors definition)
    v = myPart.vertices
    edges = myPart.edges
    myVertices = []
    for edge in edges:
        id = edge.getVertices()
        myVertices.append((v[id[0]],v[id[1]]))
    myPart.WirePolyLine(points=myVertices, mergeWire=OFF, meshable=OFF)
    #
    #------------------------------------
    # A S S E M B L Y
    #------------------------------------
    #
    # create a assembly hexagon part
    a = mdb.models['Model-1'].rootAssembly
    a.Instance(name='Hexagon', part=myPart, dependent=ON)
    #
#
#
#
#-------------------------------------------------------------------------------
#    M A I N  P A R T
#-------------------------------------------------------------------------------
#
if __name__ == '__main__':
    #
    # read data points from a file
    datapoints=readDataPoints(user_inputFile)
    #
    # define connectivity
    edges = findEdge(points=datapoints,hexLength=user_length)
    #
    # generate hexagon
    hexagon(points=edges, splitSize=user_splitSize)
    #
#-------------------------------------EOF---------------------------------------
